{% load i18n bootstrap4 %}
{% for message in messages %}
    <div class="{{ message|bootstrap_message_classes }} alert-dismissible fade show" role="alert">
        <button type="button" class="close" data-dismiss="alert" aria-label="{% trans 'close' %}">&#215;</button>
        {{ message }}
    </div>
{% endfor %}
